"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sinon = require("sinon");
const proxyquire = require("proxyquire");
const path = require("path");
const ServerTs = {
    Server: sinon.stub(),
};
describe("CLI", () => {
    let cli;
    beforeEach(() => {
        const Cli = proxyquire("./cli", { './Server': ServerTs }).Cli;
        cli = new Cli();
    });
    describe("start", () => {
        let serverObject;
        beforeEach(() => {
            serverObject = {
                start: sinon.stub().resolves(),
                addService: sinon.stub().returns(serverObject),
            };
            ServerTs.Server.returns(serverObject);
        });
        describe("without arguments", () => {
            it("should construct and start the server", () => {
                cli.start(["node", "cli.ts", "start"]);
                sinon.assert.calledWith(ServerTs.Server, sinon.match.has("dataPath", path.resolve("./data")));
                sinon.assert.calledWith(serverObject.start);
                sinon.assert.callCount(serverObject.addService, 5);
            });
        });
        describe("with data argument", () => {
            it("should construct and start the server", () => {
                cli.start(["node", "cli.ts", "start", "-d", "/tmp/mydata"]);
                sinon.assert.calledWith(ServerTs.Server, sinon.match.has("dataPath", "/tmp/mydata"));
                sinon.assert.calledWith(serverObject.start);
                sinon.assert.callCount(serverObject.addService, 5);
            });
        });
        describe("with --no-auto-keygen argument", () => {
            it("should construct and start the server", () => {
                cli.start(["node", "cli.ts", "start", "--no-auto-keygen"]);
                sinon.assert.calledWith(ServerTs.Server, sinon.match.has("autoKeyGen", false));
                sinon.assert.calledWith(serverObject.start);
                sinon.assert.callCount(serverObject.addService, 5);
            });
        });
    });
});
//# sourceMappingURL=cli.spec.js.map