"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const shared_1 = require("../shared");
const decorators_1 = require("../decorators");
const Logger_1 = require("../shared/Logger");
const SyncedRealm_1 = require("../shared/SyncedRealm");
const Server_1 = require("../Server");
const Realm = require("realm");
const fs = require("fs-extra");
const path = require("path");
let RealmFileSchema = {
    name: 'RealmFile',
    primaryKey: 'path',
    properties: {
        path: 'string',
        creatorId: 'string',
        creationDate: 'date',
        permissions: { type: 'list', objectType: 'PermissionCondition' },
        syncLabel: 'string',
    }
};
const ACCESS_LEVEL_NONE = 0;
const ACCESS_LEVEL_READ = 1;
const ACCESS_LEVEL_WRITE = 2;
const ACCESS_LEVEL_ADMIN = 3;
function accessLevelToString(accessLevel) {
    switch (accessLevel) {
        case 0: return 'none';
        case 1: return 'read';
        case 2: return 'write';
        case 3: return 'admin';
        default: throw new Error('Invalid accessLevel');
    }
}
function stringToAccessLevel(accessLevel) {
    switch (accessLevel) {
        case 'none': return 0;
        case 'read': return 1;
        case 'write': return 2;
        case 'admin': return 3;
        default: throw new Error(`Invalid accessLevel string '${accessLevel}'`);
    }
}
const PERMISSION_TYPE_ALL = 0;
const PERMISSION_TYPE_USER = 1;
const PERMISSION_TYPE_METADATA_EXACT = 2;
const PERMISSION_TYPE_METADATA_REGEX = 3;
function permissionTypeString(type) {
    switch (type) {
        case 0: return 'all';
        case 1: return 'user';
        case 2: return 'metadata_exact';
        case 3: return 'metadata_regex';
        default: throw new Error('Invalid permissionType');
    }
}
function permissionTypeForString(type) {
    switch (type) {
        case 'all': return 0;
        case 'user': return 1;
        case 'metadata_exact': return 2;
        case 'metadata_regex': return 3;
        default: throw new Error('Invalid permissionType string');
    }
}
let PermissionConditionSchema = {
    name: 'PermissionCondition',
    properties: {
        path: 'string',
        accessLevel: 'int',
        type: 'int',
        key: { type: 'string', optional: true },
        value: { type: 'string', optional: true }
    }
};
let RealmManagementService = class RealmManagementService {
    constructor(config = {}) {
        this.logger = new Logger_1.Mute();
        this.config = config;
        if (!this.config.tags || this.config.tags.length === 0) {
            this.config.tags = ["master"];
        }
        this.tags = this.config.tags;
    }
    get realm() {
        return this.syncedRealm.realm;
    }
    accessLevelForPath(path, userId) {
        let realmFile = this.validatedRealmFile(path);
        let maxLevel = ACCESS_LEVEL_NONE;
        let matching = realmFile.permissions.filtered(`type = $0 || (type = $1 && value = $2)`, PERMISSION_TYPE_ALL, PERMISSION_TYPE_USER, userId);
        matching.forEach(permission => {
            if (permission.accessLevel > maxLevel) {
                maxLevel = permission.accessLevel;
            }
        });
        return maxLevel;
    }
    setLogger(l) {
        this.logger = l;
    }
    accessToken(path, data, app_id) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!path) {
                throw new shared_1.JSONError({
                    title: `'path' parameter is required`,
                });
            }
            if (!data) {
                throw new shared_1.JSONError({
                    title: `'data' parameter is required when requesting Realm access`,
                });
            }
            const userToken = shared_1.parseTokenFromBase64RSA_SHA256(data);
            const userId = userToken.user_id;
            const isAdmin = userToken.is_admin;
            path = path.replace('~', userId);
            let realmFile = this.realm.objectForPrimaryKey('RealmFile', path);
            if (!realmFile) {
                realmFile = (yield this.createRealmFile(path, userId));
            }
            let accessLevel = isAdmin ? ACCESS_LEVEL_ADMIN : this.accessLevelForPath(path, userId);
            let access = ['download', 'upload', 'manage'];
            const appId = app_id || "io.realm.Auth";
            const expires = Math.round(Date.now() / 1000) + 600;
            const token_data = {
                identity: userId,
                appId: appId,
                access: access,
                path: path,
                timestamp: Date.now() / 1000,
                expires: expires,
                syncLabel: realmFile.syncLabel,
            };
            let token = shared_1.signToken(token_data, this.privateKey);
            return {
                access_token: {
                    token: token,
                    token_data: token_data
                }
            };
        });
    }
    find() {
        return __awaiter(this, void 0, void 0, function* () {
            const realmFiles = this.realm.objects(RealmFileSchema.name).slice();
            return realmFiles;
        });
    }
    findByPath(realmPath, accessToken) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!accessToken) {
                throw new shared_1.JSONError({
                    title: "X-Realm-Access-Token header is not valid",
                    status: 400,
                });
            }
            const parsedToken = shared_1.parseTokenFromBase64RSA_SHA256(accessToken);
            const userId = (parsedToken.identity || parsedToken.user_id);
            realmPath = realmPath.replace('~', userId);
            let realmFile = this.realm.objectForPrimaryKey(RealmFileSchema.name, realmPath);
            if (!realmFile) {
                realmFile = (yield this.createRealmFile(realmPath, userId));
            }
            return realmFile;
        });
    }
    validatedRealmFile(path) {
        if (!path) {
            throw new shared_1.JSONError({
                title: `'path' parameter is required`,
            });
        }
        let realmFile = this.realm.objectForPrimaryKey('RealmFile', path);
        if (!realmFile) {
            throw new shared_1.JSONError({
                detail: `Realm path: ${path}`,
                title: 'The provided path does not exist',
                status: 403
            });
        }
        return realmFile;
    }
    realmFileJSON(realmFile, withPermissions) {
        let json = {
            path: realmFile.path,
            creatorId: realmFile.creatorId,
            creationDate: realmFile.creationDate
        };
        if (withPermissions) {
            let permissions = this.realm.objects('PermissionCondition').filtered('path = $0', realmFile.path);
            json.permissions = permissions.map((permission) => {
                return {
                    key: permission.key,
                    value: permission.value,
                    type: permissionTypeString(permission.type),
                    accessLevel: accessLevelToString(permission.accessLevel)
                };
            });
        }
        return json;
    }
    get(incomingRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            let realmFile = this.validatedRealmFile(incomingRequest.params.path);
            return this.realmFileJSON(realmFile, true);
        });
    }
    remove(incomingRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            let realmFile = this.validatedRealmFile(incomingRequest.params.path);
            this.realm.write(() => this.realm.delete(realmFile));
        });
    }
    getPermissions(incomingRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            let realmFile = this.validatedRealmFile(incomingRequest.params.path);
            return this.realmFileJSON(realmFile, true).permissions;
        });
    }
    createPermissions(incomingRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            let realmFile = this.validatedRealmFile(incomingRequest.params.path);
            this.realm.write(() => {
                let permissions = incomingRequest.body instanceof Array ? incomingRequest.body : [incomingRequest.body];
                permissions.forEach(permission => {
                    let type = permissionTypeForString(permission.type);
                    let existing = realmFile.permissions.filtered('type = $0', type);
                    switch (type) {
                        case PERMISSION_TYPE_USER:
                            existing = existing.filtered('value = $0', permission.value);
                            break;
                        case PERMISSION_TYPE_USER:
                            existing = existing.filtered('value = $0', permission.value);
                            break;
                    }
                    let accessLevel = stringToAccessLevel(permission.accessLevel);
                    if (existing.length) {
                        if (accessLevel === ACCESS_LEVEL_NONE)
                            this.realm.delete(existing[0]);
                        else
                            existing[0].accessLevel = accessLevel;
                    }
                    else {
                        realmFile.permissions.push({
                            path: realmFile.path,
                            type: type,
                            accessLevel: accessLevel,
                            key: permission.key,
                            value: permission.value
                        });
                    }
                });
            });
            return {};
        });
    }
    start(server) {
        return __awaiter(this, void 0, void 0, function* () {
            this.discovery = server.discovery;
            this.privateKey = yield fs.readFile(server.config.privateKeyPath, 'utf-8');
            this.syncedRealm = new SyncedRealm_1.SyncedRealm(this.discovery, {
                path: path.resolve(server.config.dataPath, "realms", "realms.realm"),
                schema: [PermissionConditionSchema, RealmFileSchema],
                sync: {
                    url: "realm://default/__permissions",
                    user: Realm.Sync.User.adminUser(shared_1.generateAdminToken(this.privateKey), "realm://default/__permissions"),
                }
            });
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.syncedRealm) {
                this.syncedRealm.close();
            }
        });
    }
    createRealmFile(path, creatorId) {
        return __awaiter(this, void 0, void 0, function* () {
            const sync = yield this.discovery.find('sync', ["master"]);
            if (!sync) {
                throw new Error("Could not find a sync worker for allocation");
            }
            const labelTag = sync.tags.filter((t) => {
                return t !== "master" && t !== "slave";
            })[0] || "default";
            let realmFile;
            this.realm.write(() => {
                realmFile = this.realm.create('RealmFile', {
                    creatorId,
                    path,
                    creationDate: new Date(),
                    syncLabel: labelTag,
                });
            });
            return realmFile;
        });
    }
};
__decorate([
    decorators_1.Unmute(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Logger_1.Logger]),
    __metadata("design:returntype", void 0)
], RealmManagementService.prototype, "setLogger", null);
__decorate([
    decorators_1.Post('/access_token'),
    __param(0, decorators_1.Body('path')), __param(1, decorators_1.Body('data')), __param(2, decorators_1.Body('app_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object, String]),
    __metadata("design:returntype", Promise)
], RealmManagementService.prototype, "accessToken", null);
__decorate([
    decorators_1.Get('/files'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RealmManagementService.prototype, "find", null);
__decorate([
    decorators_1.Get('/files/:realmPath'),
    __param(0, decorators_1.Params('realmPath')),
    __param(1, decorators_1.Headers('X-Realm-Access-Token')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], RealmManagementService.prototype, "findByPath", null);
__decorate([
    decorators_1.Get('/get'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], RealmManagementService.prototype, "get", null);
__decorate([
    decorators_1.Delete('/remove'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], RealmManagementService.prototype, "remove", null);
__decorate([
    decorators_1.Get('/:path/permissions'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], RealmManagementService.prototype, "getPermissions", null);
__decorate([
    decorators_1.Post('/:path/permissions'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], RealmManagementService.prototype, "createPermissions", null);
__decorate([
    decorators_1.Start(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Server_1.Server]),
    __metadata("design:returntype", Promise)
], RealmManagementService.prototype, "start", null);
__decorate([
    decorators_1.Stop(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], RealmManagementService.prototype, "stop", null);
RealmManagementService = __decorate([
    decorators_1.BaseRoute('/realms'),
    decorators_1.ServiceName('realms'),
    __metadata("design:paramtypes", [Object])
], RealmManagementService);
exports.RealmManagementService = RealmManagementService;
//# sourceMappingURL=RealmManagementService.js.map